// 
// Copyright (C) 2007, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Enter the description of your strategy here
    /// </summary>
    [Description("Enter the description of your strategy here")]
    public class SampleHaltAdvancedStrategy : Strategy
    {
        #region Variables
		private IOrder myEntryOrder = null;
		private IOrder myExitOrder = null;
		private IOrder stopStrategyLong = null;
		private IOrder stopStrategyShort = null;
		private bool haltProcessing = false;
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			// When we determine to stop our strategy all logic in the OnBarUpdate() method will cease.
			if (haltProcessing)
				return;
			
			// After our strategy has a PnL greater than $1000 or less than -$400 we will stop our strategy
			if (Performance.AllTrades.TradesPerformance.Currency.CumProfit > 1000 
				|| Performance.AllTrades.TradesPerformance.Currency.CumProfit < -400)
			{
				// A custom method designed to close all open positions and cancel all working orders will be called.
				StopStrategy();
			}
			
			/* This print will show every time the OnBarUpdate() method is called. When strategy is stopped it will
			no longer print. */
			Print("OnBarUpdate(): " + Time[0]);
			
			if (myEntryOrder == null && Close[0] > Open[0])
			{
				/* Submits a Long Limit order at the current bar's low. This order has liveUntilCancelled set to true
				so it does not require resubmission on every new bar to keep it alive. */ 
				myEntryOrder = EnterLongLimit(0, true, 1, Low[0], "Long Limit");
			}
			
			// After 5 bars have passed since entry, exit the position
			if (BarsSinceEntry() >= 5)
			{
				// Submit our exit order
				myExitOrder = ExitLong();
			}
        }
		
		protected override void OnMarketData(MarketDataEventArgs e) 
		{
			/* When we determine to stop our strategy all logic in the OnMarketData() method will cease.
			We need to stop our strategy on every method we use; not just the OnBarUpdate() method. */
			if (haltProcessing)
				return;
			
			/* This print will show every time the OnMarketData() method is called. When the strategy is stopped it
			will no longer print. */
			Print("OnMarketData: " + e.Time + " " + e.Price + " " + e.Volume + " " + e.MarketDataType);
		}
		
		protected override void OnOrderUpdate(IOrder order)
		{
			// When we determine to stop our strategy all logic in the OnOrderUpdate() method will cease.
			if (haltProcessing)
				return;
			
			/* This print will show every time the OnOrderUpdate() method is called. When the strategy is stopped it
			will no longer print. */
			Print("OnOrderUpdate: " + order.ToString());
			
			/* Handle our entry order here. If our entry order is cancelled or rejected we want to reset myEntryOrder
			so we can enter the next time our entry signal appears. */
			if (myEntryOrder != null && myEntryOrder == order)
			{
				if (order.OrderState == OrderState.Cancelled || order.OrderState == OrderState.Rejected)
					myEntryOrder = null;
			}
			
			// Handle our exit order here.
			if (myExitOrder != null && myExitOrder == order)
			{
				// If our exit order is filled we want to reset myEntryOrder so we can submit orders again.
				if (order.OrderState == OrderState.Filled)
					myEntryOrder = null;
				
				// If our exit order encounters some problems we will resubmit the order again.
				else if (order.OrderState == OrderState.Cancelled || order.OrderState == OrderState.Rejected)
					myExitOrder = ExitLong();
			}
			
			/* Handle our haltProcessing orders here. We can identify if it is our exit orders calling that called the OnOrderUpdate()
			method by checking our IOrder objects. */
			if ((stopStrategyLong != null && stopStrategyLong == order)
				|| (stopStrategyShort != null && stopStrategyShort == order))
			{
				Print(order.ToString());
				
				// Once our exit order is filled we are safe to stop our strategy
				if (order.OrderState == OrderState.Filled)
					haltProcessing = true;
				
				// If our exit order encounters problems, resubmit the order
				else if (order.OrderState == OrderState.Cancelled || order.OrderState == OrderState.Rejected)
				{
					if (Position.MarketPosition == MarketPosition.Long)
						stopStrategyLong = ExitLong();
					else if (Position.MarketPosition == MarketPosition.Short)
						stopStrategyShort = ExitShort();
				}
			}
		}
		
		private void StopStrategy()
		{
			// If our Long Limit order is still active we will need to cancel it.
			CancelOrder(myEntryOrder);
			
			// If we have a position we will need to close the position
			if (Position.MarketPosition == MarketPosition.Long)
			{
				/* We use an IOrder here so we can monitor when our exit order becomes filled in the
				OnOrderUpdate() method. */
				stopStrategyLong = ExitLong();
			}
			else if (Position.MarketPosition == MarketPosition.Short)
			{
				/* We use an IOrder here so we can monitor when our exit order becomes filled in the
				OnOrderUpdate() method. */
				stopStrategyShort = ExitShort();
			}
			else
			{
				/* If we have no more active orders and all our positions are closed, it is now safe to
				stop the strategy. */
				haltProcessing = true;
			}
		}

        #region Properties
        #endregion
    }
}
